// one line to give the program's name and an idea of what it does.
// Copyright (C) 2005  name of Borja Snchez Zamorano
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

using System;
using System.Windows.Forms;
using System.Drawing;

namespace MonoCalendar
{
	
	public class ControlSplitter : Control
	{
		int pos = 100;
		int minPos = 0;
		int maxPos = 200;
		int orientation = 1;
		int controlBigger = 0;
		bool resizable = true;
		bool controlHidden = false;
		int separation = 5;
		bool move = false;

		public ControlSplitter()
		{
			this.SizeChanged += new System.EventHandler(this.UserControlSizeChanged);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(ControlSplitterMouseMove);
			this.MouseUp += new System.Windows.Forms.MouseEventHandler(ControlSplitterMouseUp);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(ControlSplitterMouseDown);
			this.MouseLeave += new System.EventHandler(ControlSplitterMouseLeave);
		}
		
		public int Orientation {
			set {
				orientation = value;
			}
			get {
				return orientation;
			}
		}

		public int MaxPos {
			set {
				maxPos = value;
				ChangePos(pos);
			}
			get {
				return maxPos;
			}
		}

		public int Separation {
			get {
				return separation;
			}
			set {
				separation = value;
				ChangePos(pos);
			}
		}

		public int MinPos {
			set {
				minPos = value;
				ChangePos(pos);
			}
			get {
				return minPos;
			}
		}

		public int Pos {
			set {
				pos = value;
				ChangePos(pos);
			}
			get {
				return pos;
			}
		}

		public bool Resizable {
			set {
				resizable = value;
			}
			get {
				return resizable;
			}
		}

		public bool ControlHidden {
			set {
				if (value)
					this.Controls[1-controlBigger].Hide();
				else
					this.Controls[1-controlBigger].Show();
				controlHidden = value;
				ChangePos(pos);
			}
			get {
				return controlHidden;
			}
		}

		public int ControlBigger {
			set {
				controlBigger = value;
			}
			get {
				return controlBigger;
			}
		}
		
		void UserControlSizeChanged(object sender, System.EventArgs e)
		{
			ChangePos(pos);
		}
		
		void ChangePos(int x2)
		{
			if (controlHidden) {
				this.Controls[controlBigger].SetBounds(0, 0, this.ClientRectangle.Width, this.ClientRectangle.Height);
				return;
			}

			pos = x2;
			if (pos < minPos) pos = minPos;
			if (pos > maxPos) pos = maxPos;
			int pos2 = pos;
			if (controlBigger==0) {
				if (orientation == 1) {
					pos2 = this.ClientRectangle.Width - pos;
				} else {
					pos2 = this.ClientRectangle.Height - pos;
				}
			}
			
			if (this.Controls.Count < 2) return;
			
			this.Controls[0].SuspendLayout();
			this.Controls[1].SuspendLayout();
			if (orientation == 1)
			{
				this.Controls[0].Top = 0;
				this.Controls[0].Left = 0;
				this.Controls[0].Height = this.ClientRectangle.Height;
				this.Controls[0].Width = pos2-(separation/2);
				this.Controls[1].Top = 0;
				this.Controls[1].Left = this.Controls[0].Width + separation;
				this.Controls[1].Height = this.ClientRectangle.Height;
				this.Controls[1].Width = this.ClientRectangle.Width - this.Controls[1].Left;
			}
			else
			{
				this.Controls[0].Top = 0;
				this.Controls[0].Left = 0;
				this.Controls[0].Width = this.ClientRectangle.Width;
				this.Controls[0].Height = pos2-(separation/2);
				this.Controls[1].Top = this.Controls[0].Height + separation;
				this.Controls[1].Left = 0;
				this.Controls[1].Width = this.ClientRectangle.Width;
				this.Controls[1].Height = this.ClientRectangle.Height - this.Controls[1].Top;
			}
			this.Controls[0].ResumeLayout(true);
			this.Controls[1].ResumeLayout(true);
		}
		
		bool sizing = false;
		void ControlSplitterMouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (!resizable) return;

			switch (e.Button)
			{
				case System.Windows.Forms.MouseButtons.Left:
					if (move) {
						if (orientation == 1)
							ChangePos((controlBigger==0)?this.ClientRectangle.Width-e.X:e.X);
						else
							ChangePos((controlBigger==0)?this.ClientRectangle.Height-e.Y:e.Y);
						if (!sizing) if (BeginResize != null) BeginResize(this, System.EventArgs.Empty);
						sizing = true;
					}
					break;
			}
			if (orientation == 1)
				this.Cursor = System.Windows.Forms.Cursors.SizeWE;
			else
				this.Cursor = System.Windows.Forms.Cursors.SizeNS;
		}

		private void ControlSplitterMouseLeave(object sender, System.EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		private void ControlSplitterMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (this.Cursor != System.Windows.Forms.Cursors.Default)
			{
				this.Capture = true;
				move = true;
			}
		}

		private void ControlSplitterMouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == System.Windows.Forms.MouseButtons.Left) {
				this.Cursor = System.Windows.Forms.Cursors.Default;
				if (sizing) {
					move = false;
					this.Capture = false;
					if (FinishResize != null) FinishResize(this, System.EventArgs.Empty);
					sizing = false;
				}
			}
		}

		public event EventHandler FinishResize;
		public event EventHandler BeginResize;
	}
}
